==> slashdb_diag-1.0.0-py3-none-any.whl: slashdb_diag/nginx.py <==
from typing import Dict, Iterable, Union, Callable, Optional
from collections import defaultdict
from datetime import datetime
from itertools import islice
import re

# how the output should be formatted
DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S"

# used if we can't get format from nginx.conf
# should be up to date with dist-configs/default/nginx.conf
NGINX_FORMAT_FALLBACK = (
    "$time_iso8601mod | $msec_int | nginx | nginx | INFO "
    "| $pid |  | $request_id  | $request_method | $request_uri "
    "| $status | $request_time  | $request_length | $bytes_sent "
    "| $remote_addr | $http_referer | $http_user_agent |  |  | "
)


def _get_key(key, default=None):
    def inner(d: Dict):
        return d.get(key, default)

    return inner


PARAM_MAPPING: Dict[str, Callable[[Dict], str]] = defaultdict(
    lambda: lambda _: "",
    {
        "$time_iso8601mod": lambda params: params["datetime"].strftime(DATETIME_FORMAT),
        "$msec_int": lambda params: str(int(params["datetime"].timestamp() * 1000.0)),
        "$level": lambda params: params.get("level", "error").upper(),
        "$pid": _get_key("pid", "N/A"),
        "$request_uri": lambda params: f'"{params.get("request_uri", "")}"',
        "$request_method": _get_key("request_method", ""),
        "$remote_addr": _get_key("client", ""),
        "$http_referer": _get_key("http_referrer", ""),
    },
)


def fill_out_entry(params: Dict, format_str: Optional[str] = None):
    if format_str is None:
        format_str = NGINX_FORMAT_FALLBACK

    # level param placeholder is not present in access logs,
    # so we inject it manually
    format_str = format_str.replace("INFO", "$level")

    format_params = re.findall(r"\$\w*", format_str)
    log_line = format_str
    for format_param in format_params:
        log_line = log_line.replace(format_param, PARAM_MAPPING[format_param](params))

    log_line += params.get("msg", "")
    return log_line


def check_if_nginx_error(log_line: str) -> bool:
    try:
        parse_nginx_error_datetime(log_line)
    except ValueError:
        return False
    return True


def try_fix_nginx_error(log_line: str, format_str: Optional[str] = None) -> str:
    if not check_if_nginx_error(log_line):
        return log_line
    params = parse_nginx_error(log_line)
    return fill_out_entry(params, format_str) + "\n"


def parse_nginx_error_datetime(log_line: str) -> datetime:
    # 2023/01/16 15:04:04 ...
    date_time_str = "T".join(log_line.split()[:2])
    return datetime.strptime(date_time_str, "%Y/%m/%dT%H:%M:%S")


def parse_nginx_error(log_line: str) -> Dict:
    params = {}
    params["datetime"] = parse_nginx_error_datetime(log_line)
    params["msg"] = " ".join(log_line.split()[4:])

    # get level, pid and tid
    if match := re.match(r"\[(\w+)\] (\d+)#(\d+):.*", " ".join(log_line.split()[2:])):
        params["level"] = match.group(1)
        params["pid"] = match.group(2)
        params["tid"] = match.group(3)

    params.update(parse_nginx_error_parameters(params["msg"]))

    return params


def parse_nginx_error_parameters(msg: str) -> Dict[str, str]:
    def parse_parameter(s: str) -> (str, str):
        k_v = tuple(map(str.strip, s.split(":", 1)))
        if len(k_v) == 0:
            return ("", "")
        elif len(k_v) == 1:
            return (k_v[0], "")
        else:
            return k_v

    def split_params(s: str) -> Iterable[str]:
        """String-aware comma splitting"""
        in_str = False
        buf = ""
        for c in s:
            if c == '"':
                in_str = not in_str
                buf += c
            elif c == "," and not in_str:
                yield buf
                buf = ""
            else:
                buf += c
        yield buf

    params = dict(map(parse_parameter, islice(split_params(msg), 1, None)))
    if "request" in params:
        request = params.pop("request")
        params.update(parse_nginx_error_request(request))
    return params


def parse_nginx_error_request(request: str) -> Dict[str, str]:
    method, uri, _proto = request.split()
    method = method.strip('"')
    return {"request_method": method, "request_uri": uri}

==> slashdb_diag-1.1.0-py3-none-any.whl: slashdb_diag/nginx.py <==
from typing import Dict, Iterable, Callable, Optional, Tuple, Any
from collections import defaultdict
from datetime import datetime
from itertools import islice
import re

# how the output should be formatted
DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S"

# used if we can't get format from nginx.conf
# should be up to date with dist-configs/default/nginx.conf
NGINX_FORMAT_FALLBACK = (
    "$time_iso8601mod | $msec_int | nginx | nginx | INFO "
    "| $pid |  | $request_id  | $request_method | $request_uri "
    "| $status | $request_time  | $request_length | $bytes_sent "
    "| $remote_addr | $http_referer | $http_user_agent |  |  | "
)


def _get_key(key, default=None):
    def inner(d: Dict):
        return d.get(key, default)

    return inner


PARAM_MAPPING: Dict[str, Callable[[Dict], str]] = defaultdict(
    lambda: lambda _: "",
    {
        "$time_iso8601mod": lambda params: params["datetime"].strftime(DATETIME_FORMAT),
        "$msec_int": lambda params: str(int(params["datetime"].timestamp() * 1000.0)),
        "$level": lambda params: params.get("level", "error").upper(),
        "$pid": _get_key("pid", "N/A"),
        "$request_uri": lambda params: f'"{params.get("request_uri", "")}"',
        "$request_method": _get_key("request_method", ""),
        "$remote_addr": _get_key("client", ""),
        "$http_referer": _get_key("http_referrer", ""),
    },
)


def fill_out_entry(params: Dict, format_str: Optional[str] = None):
    if format_str is None:
        format_str = NGINX_FORMAT_FALLBACK

    # level param placeholder is not present in access logs,
    # so we inject it manually
    format_str = format_str.replace("INFO", "$level")

    format_params = re.findall(r"\$\w*", format_str)
    log_line = format_str
    for format_param in format_params:
        log_line = log_line.replace(format_param, PARAM_MAPPING[format_param](params))

    log_line += params.get("msg", "")
    return log_line


def check_if_nginx_error(log_line: str) -> bool:
    try:
        parse_nginx_error_datetime(log_line)
    except ValueError:
        return False
    return True


def try_fix_nginx_error(log_line: str, format_str: Optional[str] = None) -> str:
    if not check_if_nginx_error(log_line):
        return log_line
    params = parse_nginx_error(log_line)
    return fill_out_entry(params, format_str) + "\n"


def parse_nginx_error_datetime(log_line: str) -> datetime:
    # 2023/01/16 15:04:04 ...
    date_time_str = "T".join(log_line.split()[:2])
    return datetime.strptime(date_time_str, "%Y/%m/%dT%H:%M:%S")


def parse_nginx_error(log_line: str) -> Dict[str, Any]:
    params: Dict[str, Any] = {}
    params["datetime"] = parse_nginx_error_datetime(log_line)
    params["msg"] = " ".join(log_line.split()[4:])

    # get level, pid and tid
    if match := re.match(r"\[(\w+)\] (\d+)#(\d+):.*", " ".join(log_line.split()[2:])):
        params["level"] = match.group(1)
        params["pid"] = match.group(2)
        params["tid"] = match.group(3)

    params.update(parse_nginx_error_parameters(params["msg"]))

    return params


def parse_nginx_error_parameters(msg: str) -> Dict[str, str]:
    def parse_parameter(s: str) -> Tuple[str, str]:
        k_v = tuple(map(str.strip, s.split(":", 1)))
        if len(k_v) == 0:
            return ("", "")
        elif len(k_v) == 1:
            return (k_v[0], "")
        else:
            return k_v

    def split_params(s: str) -> Iterable[str]:
        """String-aware comma splitting"""
        in_str = False
        buf = ""
        for c in s:
            if c == '"':
                in_str = not in_str
                buf += c
            elif c == "," and not in_str:
                yield buf
                buf = ""
            else:
                buf += c
        yield buf

    params = dict(map(parse_parameter, islice(split_params(msg), 1, None)))
    if "request" in params:
        request = params.pop("request")
        params.update(parse_nginx_error_request(request))
    return params


def parse_nginx_error_request(request: str) -> Dict[str, str]:
    method, uri, _proto = request.split()
    method = method.strip('"')
    return {"request_method": method, "request_uri": uri}
